@@ -55,6 +55,10 @@ h8300_scan (info, string)
     {
       return (info->mach == bfd_mach_h8300h);
     }
+  else if (*string == 's' || *string == 'S')
+    {
+      return (info->mach == bfd_mach_h8300s);
+    }
   else
     {
       return info->mach == bfd_mach_h8300;
@@ -78,7 +82,6 @@ compatible (in, out)
     return in;
 }
 
-#define H8300H_INFO_STRUCT bfd_h8300_arch
 static const bfd_arch_info_type h8300_info_struct =
 {
   16,				/* 16 bits in a word */
@@ -96,7 +99,7 @@ static const bfd_arch_info_type h8300_info_struct =
   0,
 };
 
-const bfd_arch_info_type H8300H_INFO_STRUCT =
+static const bfd_arch_info_type h8300h_info_struct =
 {
   32,				/* 32 bits in a word */
   32,				/* 32 bits in an address */
@@ -113,5 +116,19 @@ const bfd_arch_info_type H8300H_INFO_STRUCT =
   &h8300_info_struct,
 };
 
-
-#undef H8300_INFO_STRUCT
+const bfd_arch_info_type bfd_h8300_arch =
+{
+  32,				/* 32 bits in a word */
+  32,				/* 32 bits in an address */
+  8,				/* 8 bits in a byte */
+  bfd_arch_h8300,
+  bfd_mach_h8300s,
+  "h8300s",			/* arch_name  */
+  "h8300s",			/* printable name */
+  1,
+  false,			/* the default machine */
+  compatible,
+  h8300_scan,
+/*    local_bfd_reloc_type_lookup, */
+  &h8300h_info_struct,
+};
