@@ -1,5 +1,5 @@
 /* BFD support for Sparc binaries under LynxOS.
-   Copyright (C) 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1990, 91, 92, 93, 94, 95, 1997 Free Software Foundation, Inc.
 
 This file is part of BFD, the Binary File Descriptor library.
 
@@ -15,15 +15,15 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #if 0
 #define BYTES_IN_WORD 4
 #define N_SHARED_LIB(x) 0
 
 #define TEXT_START_ADDR 0
-#define PAGE_SIZE 4096
-#define SEGMENT_SIZE PAGE_SIZE
+#define TARGET_PAGE_SIZE 4096
+#define SEGMENT_SIZE TARGET_PAGE_SIZE
 #define DEFAULT_ARCH bfd_arch_sparc
 
 #endif
@@ -228,7 +228,7 @@ sparclynx_set_sizes (abfd)
 
 static CONST struct aout_backend_data sparclynx_aout_backend =
 {
-  0, 1, 1, 0, sparclynx_set_sizes, 0,
+  0, 1, 0, 1, 0, sparclynx_set_sizes, 0,
   0,				/* add_dynamic_symbols */
   0,				/* add_one_symbol */
   0,				/* link_dynamic_object */
@@ -248,7 +248,7 @@ static CONST struct aout_backend_data sparclynx_aout_backend =
 
 #define TARGET_IS_BIG_ENDIAN_P
 
-#ifdef HOST_LYNX
+#ifdef LYNX_CORE
 
 char *lynx_core_file_failing_command ();
 int lynx_core_file_failing_signal ();
@@ -260,6 +260,6 @@ const bfd_target *lynx_core_file_p ();
 #define	MY_core_file_matches_executable_p lynx_core_file_matches_executable_p
 #define	MY_core_file_p lynx_core_file_p
 
-#endif /* HOST_LYNX */
+#endif /* LYNX_CORE */
 
 #include "aout-target.h"
