@@ -839,7 +839,8 @@ sunos_create_dynamic_sections (abfd, info, needed)
       sunos_hash_table (info)->dynamic_sections_created = true;
     }
 
-  if (needed && ! sunos_hash_table (info)->dynamic_sections_needed)
+  if ((needed && ! sunos_hash_table (info)->dynamic_sections_needed)
+      || info->shared)
     {
       bfd *dynobj;
 
@@ -963,7 +964,10 @@ sunos_add_dynamic_symbols (abfd, info, symsp, sym_countp, stringsp)
       unsigned long name, flags;
       unsigned short major_vno, minor_vno;
       struct bfd_link_needed_list *needed, **pp;
+      char *namebuf, *p;
+      size_t alc;
       bfd_byte b;
+      char *namecopy;
 
       if (bfd_seek (abfd, need, SEEK_SET) != 0
 	  || bfd_read (buf, 1, 16, abfd) != 16)
@@ -985,33 +989,91 @@ sunos_add_dynamic_symbols (abfd, info, symsp, sym_countp, stringsp)
       needed->by = abfd;
 
       /* We return the name as [-l]name[.maj][.min].  */
+      alc = 30;
+      namebuf = (char *) bfd_malloc (alc + 1);
+      if (namebuf == NULL)
+	return false;
+      p = namebuf;
 
       if ((flags & 0x80000000) != 0)
-	bfd_alloc_grow (abfd, "-l", 2);
+	{
+	  *p++ = '-';
+	  *p++ = 'l';
+	}
       if (bfd_seek (abfd, name, SEEK_SET) != 0)
-	return false;
+	{
+	  free (namebuf);
+	  return false;
+	}
+
       do
 	{
 	  if (bfd_read (&b, 1, 1, abfd) != 1)
-	    return false;
-	  bfd_alloc_grow (abfd, &b, 1);
+	    {
+	      free (namebuf);
+	      return false;
+	    }
+
+	  if (p - namebuf >= alc)
+	    {
+	      char *n;
+
+	      alc *= 2;
+	      n = (char *) bfd_realloc (namebuf, alc + 1);
+	      if (n == NULL)
+		{
+		  free (namebuf);
+		  return false;
+		}
+	      p = n + (p - namebuf);
+	      namebuf = n;
+	    }
+
+	  *p++ = b;
 	}
       while (b != '\0');
-      if (major_vno != 0)
-	{
-	  char verbuf[30];
 
-	  sprintf (verbuf, ".%d", major_vno);
-	  bfd_alloc_grow (abfd, verbuf, strlen (verbuf));
-	  if (minor_vno != 0)
+      if (major_vno == 0)
+	*p = '\0';
+      else
+	{
+	  char majbuf[30];
+	  char minbuf[30];
+
+	  sprintf (majbuf, ".%d", major_vno);
+	  if (minor_vno == 0)
+	    minbuf[0] = '\0';
+	  else
+	    sprintf (minbuf, ".%d", minor_vno);
+
+	  if ((p - namebuf) + strlen (majbuf) + strlen (minbuf) >= alc)
 	    {
-	      sprintf (verbuf, ".%d", minor_vno);
-	      bfd_alloc_grow (abfd, verbuf, strlen (verbuf));
+	      char *n;
+
+	      alc = (p - namebuf) + strlen (majbuf) + strlen (minbuf);
+	      n = (char *) bfd_realloc (namebuf, alc + 1);
+	      if (n == NULL)
+		{
+		  free (namebuf);
+		  return false;
+		}
+	      p = n + (p - namebuf);
+	      namebuf = n;
 	    }
+
+	  strcpy (p, majbuf);
+	  strcat (p, minbuf);
 	}
-      needed->name = bfd_alloc_finish (abfd);
-      if (needed->name == NULL)
-	return false;
+
+      namecopy = bfd_alloc (abfd, strlen (namebuf) + 1);
+      if (namecopy == NULL)
+	{
+	  free (namebuf);
+	  return false;
+	}
+      strcpy (namecopy, namebuf);
+      free (namebuf);
+      needed->name = namecopy;
 
       needed->next = NULL;
 
