@@ -1,5 +1,5 @@
 /* ar.c - Archive modify and extract.
-   Copyright 1991, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright 1991, 92, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
 
 This file is part of GNU Binutils.
 
@@ -54,7 +54,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 struct ar_hdr *
   bfd_special_undocumented_glue PARAMS ((bfd * abfd, char *filename));
 
-/* Forward declarations */
+/* Static declarations */
+
+static void
+mri_emul PARAMS ((void));
 
 static const char *
 normalize PARAMS ((const char *, bfd *));
@@ -142,12 +145,15 @@ enum pos
     pos_default, pos_before, pos_after, pos_end
   } postype = pos_default;
 
+static bfd **
+get_pos_bfd PARAMS ((bfd **, enum pos));
+
 /* Whether to truncate names of files stored in the archive.  */
 static boolean ar_truncate = false;
 
 int interactive = 0;
 
-void
+static void
 mri_emul ()
 {
   interactive = isatty (fileno (stdin));
@@ -340,6 +346,8 @@ main (argc, argv)
   START_PROGRESS (program_name, 0);
 
   bfd_init ();
+  set_default_bfd_target ();
+
   show_version = 0;
 
   xatexit (remove_output);
@@ -994,7 +1002,7 @@ write_archive (iarch)
    into when altering.  DEFAULT_POS should be how to interpret pos_default,
    and should be a pos value.  */
 
-bfd **
+static bfd **
 get_pos_bfd (contents, default_pos)
      bfd **contents;
      enum pos default_pos;
