@@ -15,7 +15,7 @@
 
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by the Center for Software Science at the University of Utah
    and by Cygnus Support.  */
@@ -26,7 +26,7 @@
 #define OBJ_SOM 1
 
 #include <bfd.h>
-#include "../bfd/som.h"
+#include "bfd/som.h"
 #include "targ-cpu.h"
 
 #ifndef FALSE
@@ -55,7 +55,7 @@ extern void obj_som_init_stab_section PARAMS ((segT));
 /* Stabs go in a separate sections.  GDB expects to find them in sections
    with the names $GDB_SYMBOLS$ and $GDB_STRINGS$ rather than .stab and
    .stabstr.  */
-#define SEPARATE_STAB_SECTIONS
+#define SEPARATE_STAB_SECTIONS 1
 #define STAB_SECTION_NAME "$GDB_SYMBOLS$"
 #define STAB_STRING_SECTION_NAME "$GDB_STRINGS$"
 
