@@ -37,6 +37,18 @@
 #define TC_RELOC(X,Y) (Y)
 #endif
 
+static unsigned long mode_from_disp_size PARAMS ((unsigned long));
+static int fits_in_signed_byte PARAMS ((long));
+static int fits_in_unsigned_byte PARAMS ((long));
+static int fits_in_unsigned_word PARAMS ((long));
+static int fits_in_signed_word PARAMS ((long));
+static int smallest_imm_type PARAMS ((long));
+static void set_16bit_code_flag PARAMS ((int));
+#ifdef BFD_ASSEMBLER
+static bfd_reloc_code_real_type reloc
+  PARAMS ((int, int, bfd_reloc_code_real_type));
+#endif
+
 /* 'md_assemble ()' gathers together information and puts it into a
    i386_insn. */
 
@@ -104,7 +116,7 @@ typedef struct _i386_insn i386_insn;
 
 /* This array holds the chars that always start a comment.  If the
    pre-processor is disabled, these aren't very useful */
-#if defined (TE_I386AIX) || defined (OBJ_ELF)
+#if defined (TE_I386AIX) || defined (OBJ_ELF) || defined (OBJ_MAYBE_ELF)
 const char comment_chars[] = "#/";
 #else
 const char comment_chars[] = "#";
@@ -118,7 +130,7 @@ const char comment_chars[] = "#";
    #NO_APP at the beginning of its output. */
 /* Also note that comments started like this one will always work if
    '/' isn't otherwise defined.  */
-#if defined (TE_I386AIX) || defined (OBJ_ELF)
+#if defined (TE_I386AIX) || defined (OBJ_ELF) || defined (OBJ_MAYBE_ELF)
 const char line_comment_chars[] = "";
 #else
 const char line_comment_chars[] = "/";
@@ -409,7 +421,8 @@ smallest_imm_type (num)
 	  : (Imm32));
 }				/* smallest_imm_type() */
 
-void set_16bit_code_flag(new_16bit_code_flag)
+static void
+set_16bit_code_flag (new_16bit_code_flag)
 	int new_16bit_code_flag;
 {
   flag_16bit_code = new_16bit_code_flag;
@@ -577,10 +590,13 @@ md_begin ()
       operand_chars[(unsigned char) *p] = *p;
   }
 
-#ifdef OBJ_ELF
-  record_alignment (text_section, 2);
-  record_alignment (data_section, 2);
-  record_alignment (bss_section, 2);
+#if defined (OBJ_ELF) || defined (OBJ_MAYBE_ELF)
+  if (OUTPUT_FLAVOR == bfd_target_elf_flavour)
+    {
+      record_alignment (text_section, 2);
+      record_alignment (data_section, 2);
+      record_alignment (bss_section, 2);
+    }
 #endif
 }
 
@@ -696,7 +712,6 @@ type_names[] =
   { Reg8, "r8" },
   { Reg16, "r16" },
   { Reg32, "r32" },
-  { RegMMX, "rMMX" },
   { Imm8, "i8" },
   { Imm8S, "i8s" },
   { Imm16, "i16" },
@@ -774,14 +789,6 @@ reloc (size, pcrel, other)
 	  pcrel ? "pc-relative " : "");
   return BFD_RELOC_NONE;
 }
-#else
-#define reloc(SIZE,PCREL,OTHER)	0
-#define BFD_RELOC_32		0
-#define BFD_RELOC_32_PCREL	0
-#define BFD_RELOC_386_PLT32	0
-#define BFD_RELOC_386_GOT32	0
-#define BFD_RELOC_386_GOTOFF	0
-#endif
 
 /*
  * Here we decide which fixups can be adjusted to make them relative to
@@ -797,20 +804,24 @@ tc_i386_fix_adjustable(fixP)
   /* Prevent all adjustments to global symbols. */
   if (S_IS_EXTERN (fixP->fx_addsy))
     return 0;
-#ifdef BFD_ASSEMBLER
   if (S_IS_WEAK (fixP->fx_addsy))
     return 0;
-#endif /* BFD_ASSEMBLER */
 #endif /* ! defined (OBJ_AOUT) */
-#ifdef BFD_ASSEMBLER
   /* adjust_reloc_syms doesn't know about the GOT */
   if (fixP->fx_r_type == BFD_RELOC_386_GOTOFF
       || fixP->fx_r_type == BFD_RELOC_386_PLT32
       || fixP->fx_r_type == BFD_RELOC_386_GOT32)
     return 0;
-#endif
   return 1;
 }
+#else
+#define reloc(SIZE,PCREL,OTHER)	0
+#define BFD_RELOC_32		0
+#define BFD_RELOC_32_PCREL	0
+#define BFD_RELOC_386_PLT32	0
+#define BFD_RELOC_386_GOT32	0
+#define BFD_RELOC_386_GOTOFF	0
+#endif
 
 /* This is the guts of the machine-dependent assembler.  LINE points to a
    machine dependent instruction.  This function is supposed to emit
@@ -1330,6 +1341,13 @@ md_assemble (line)
 	    i.reg_operands = 2;
 	  }
 
+	/* The clr %reg instruction is converted into xor %reg, %reg.  */
+	if (t->opcode_modifier & iclrKludge)
+	  {
+	    i.regs[1] = i.regs[0];
+	    i.reg_operands = 2;
+	  }
+
 	/* Certain instructions expect the destination to be in the i.rm.reg
 	   field.  This is by far the exceptional case.  For these
 	   instructions, if the source operand is a register, we must reverse
@@ -1677,7 +1695,7 @@ md_assemble (line)
 		       ? ENCODE_RELAX_STATE (UNCOND_JUMP, BYTE)
 		       : ENCODE_RELAX_STATE (COND_JUMP, BYTE)),
 		      i.disps[0]->X_add_symbol,
-		      (long) n, p);
+		      (offsetT) n, p);
 	  }
       }
     else if (t->opcode_modifier & (JumpByte | JumpDword))
@@ -2654,11 +2672,13 @@ md_apply_fix3 (fixP, valp, seg)
   if (fixP->fx_r_type == BFD_RELOC_32_PCREL && fixP->fx_addsy)
     {
 #ifndef OBJ_AOUT
-      value += fixP->fx_where + fixP->fx_frag->fr_address;
+      if (OUTPUT_FLAVOR == bfd_target_elf_flavour)
+	value += fixP->fx_where + fixP->fx_frag->fr_address;
 #endif
-#ifdef OBJ_ELF
-      if (S_GET_SEGMENT (fixP->fx_addsy) == seg
-	  || (fixP->fx_addsy->bsym->flags & BSF_SECTION_SYM) != 0)
+#if defined (OBJ_ELF) || defined (OBJ_MAYBE_ELF)
+      if (OUTPUT_FLAVOR == bfd_target_elf_flavour
+	  && (S_GET_SEGMENT (fixP->fx_addsy) == seg
+	      || (fixP->fx_addsy->bsym->flags & BSF_SECTION_SYM) != 0))
 	{
 	  /* Yes, we add the values in twice.  This is because
 	     bfd_perform_relocation subtracts them out again.  I think
@@ -2671,8 +2691,9 @@ md_apply_fix3 (fixP, valp, seg)
 
   /* Fix a few things - the dynamic linker expects certain values here,
      and we must not dissappoint it. */
-#ifdef OBJ_ELF
-  if (fixP->fx_addsy)
+#if defined (OBJ_ELF) || defined (OBJ_MAYBE_ELF)
+  if (OUTPUT_FLAVOR == bfd_target_elf_flavour
+      && fixP->fx_addsy)
     switch(fixP->fx_r_type) {
     case BFD_RELOC_386_PLT32:
       /* Make the jump instruction point to the address of the operand.  At
@@ -2861,7 +2882,7 @@ md_parse_option (c, arg)
       flag_do_long_jump = 1;
       break;
 
-#ifdef OBJ_ELF
+#if defined (OBJ_ELF) || defined (OBJ_MAYBE_ELF)
       /* -k: Ignore for FreeBSD compatibility.  */
     case 'k':
       break;
@@ -2890,9 +2911,32 @@ md_show_usage (stream)
   fprintf (stream, "\
 -m			do long jump\n");
 }
-
-/* We have no need to default values of symbols.  */
 
+#ifdef BFD_ASSEMBLER
+#ifdef OBJ_MAYBE_ELF
+#ifdef OBJ_MAYBE_COFF
+
+/* Pick the target format to use.  */
+
+const char  *
+i386_target_format ()
+{
+  switch (OUTPUT_FLAVOR)
+    {
+    case bfd_target_coff_flavour:
+      return "coff-i386";
+    case bfd_target_elf_flavour:
+      return "elf32-i386";
+    default:
+      abort ();
+      return NULL;
+    }
+}
+
+#endif /* OBJ_MAYBE_COFF */
+#endif /* OBJ_MAYBE_ELF */
+#endif /* BFD_ASSEMBLER */
+
 /* ARGSUSED */
 symbolS *
 md_undefined_symbol (name)
@@ -3002,9 +3046,7 @@ tc_gen_reloc (section, fixp)
       && fixp->fx_addsy == GOT_symbol)
     code = BFD_RELOC_386_GOTPC;
 
-  rel = (arelent *) bfd_alloc_by_size_t (stdoutput, sizeof (arelent));
-  if (rel == NULL)
-    as_fatal ("Out of memory");
+  rel = (arelent *) xmalloc (sizeof (arelent));
   rel->sym_ptr_ptr = &fixp->fx_addsy->bsym;
   rel->address = fixp->fx_frag->fr_address + fixp->fx_where;
   if (fixp->fx_pcrel)
