@@ -1,5 +1,5 @@
 /* tc-i386.h -- Header file for tc-i386.c
-   Copyright (C) 1989, 92, 93, 94, 95, 1996 Free Software Foundation.
+   Copyright (C) 1989, 92, 93, 94, 95, 96, 1997 Free Software Foundation.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -14,12 +14,19 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with GAS; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 #ifndef TC_I386
 #define TC_I386 1
 
+#ifdef ANSI_PROTOTYPES
+struct fix;
+#endif
+
+#define TARGET_BYTES_BIG_ENDIAN	0
+
 #ifdef TE_LYNX
 #define TARGET_FORMAT		"coff-i386-lynx"
 #endif
@@ -36,6 +43,7 @@
 	   (X) != BFD_RELOC_386_GOTPC) ? Y : X)
 
 #define tc_fix_adjustable(X)  tc_i386_fix_adjustable(X)
+extern int tc_i386_fix_adjustable PARAMS ((struct fix *));
 
 /* This is the relocation type for direct references to GLOBAL_OFFSET_TABLE.
  * It comes up in complicated expressions such as 
@@ -59,7 +67,6 @@
    && (FIX)->fx_r_type != BFD_RELOC_386_GOTPC)
 
 #define TARGET_ARCH		bfd_arch_i386
-#define TARGET_BYTES_BIG_ENDIAN	0
 
 #ifdef OBJ_AOUT
 #ifdef TE_NetBSD
@@ -74,6 +81,9 @@
 #ifdef TE_Mach
 #define TARGET_FORMAT		"a.out-mach3"
 #endif
+#ifdef TE_DYNIX
+#define TARGET_FORMAT		"a.out-i386-dynix"
+#endif
 #ifndef TARGET_FORMAT
 #define TARGET_FORMAT		"a.out-i386"
 #endif
@@ -83,6 +93,13 @@
 #define TARGET_FORMAT		"elf32-i386"
 #endif
 
+#ifdef OBJ_MAYBE_ELF
+#ifdef OBJ_MAYBE_COFF
+extern const char *i386_target_format PARAMS ((void));
+#define TARGET_FORMAT i386_target_format ()
+#endif
+#endif
+
 #else /* ! BFD_ASSEMBLER */
 
 /* COFF STUFF */
@@ -93,7 +110,7 @@
 #define TC_COUNT_RELOC(x) ((x)->fx_addsy || (x)->fx_r_type==7)
 #define TC_FORCE_RELOCATION(x) ((x)->fx_r_type==7)
 #define TC_COFF_FIX2RTYPE(fixP) tc_coff_fix2rtype(fixP)
-extern short tc_coff_fix2rtype ();
+extern short tc_coff_fix2rtype PARAMS ((struct fix *));
 #define TC_COFF_SIZEMACHDEP(frag) tc_coff_sizemachdep(frag)
 extern int tc_coff_sizemachdep PARAMS ((fragS *frag));
 #define SUB_SEGMENT_ALIGN(SEG) 2
@@ -121,14 +138,16 @@ extern int tc_coff_sizemachdep PARAMS ((fragS *frag));
 #endif
 #define tc_coff_symbol_emit_hook(a)	;	/* not used */
 
+#ifndef BFD_ASSEMBLER
 #ifndef OBJ_AOUT
 #ifndef TE_PE
 /* Local labels starts with .L */
 #define LOCAL_LABEL(name) (name[0] == '.' \
 		 && (name[1] == 'L' || name[1] == 'X' || name[1] == '.'))
-#define FAKE_LABEL_NAME ".L0\001"
 #endif
 #endif
+#endif
+
 #define LOCAL_LABELS_FB 1
 
 #define tc_aout_pre_write_hook(x)	{;}	/* not used */
@@ -184,8 +203,7 @@ extern int tc_coff_sizemachdep PARAMS ((fragS *frag));
 #define Reg8    0x1		/* 8 bit reg */
 #define Reg16   0x2		/* 16 bit reg */
 #define Reg32   0x4		/* 32 bit reg */
-#define RegMMX 0x40000000
-#define Reg     (Reg8|Reg16|Reg32|RegMMX)	/* gen'l register */
+#define Reg     (Reg8|Reg16|Reg32)	/* gen'l register */
 #define WordReg (Reg16|Reg32)	/* for push/pop operands */
 /* immediate */
 #define Imm8    0x8		/* 8 bit immediate */
@@ -277,6 +295,7 @@ typedef struct
 #define ReverseRegRegmem 0x10000
 #define Data16 0x20000		/* needs data prefix if in 32-bit mode */
 #define Data32 0x40000		/* needs data prefix if in 16-bit mode */
+#define iclrKludge 0x80000	/* used to convert clr to xor */
 
   /* (opcode_modifier & COMES_IN_ALL_SIZES) is true if the
      instuction comes in byte, word, and dword sizes and is encoded into
@@ -364,7 +383,7 @@ base_index_byte;
 #endif
 
 #ifdef BFD_ASSEMBLER
-void i386_validate_fix ();
+void i386_validate_fix PARAMS ((struct fix *));
 #define TC_VALIDATE_FIX(FIXP,SEGTYPE,SKIP) i386_validate_fix(FIXP)
 #endif
 
@@ -378,14 +397,14 @@ extern const struct relax_type md_relax_table[];
 
 extern int flag_16bit_code;
 
-#define md_do_align(n, fill, len, around)				\
+#define md_do_align(n, fill, len, max, around)				\
 if ((n) && !need_pass_2							\
     && (!(fill) || ((char)*(fill) == (char)0x90 && (len) == 1))		\
     && now_seg != data_section && now_seg != bss_section)		\
   {									\
     char *p;								\
-    p = frag_var(rs_align_code, 15, 1, (relax_substateT) 0,		\
-		 (symbolS *) 0, (long) (n), (char *) 0);		\
+    p = frag_var (rs_align_code, 15, 1, (relax_substateT) max,		\
+		  (symbolS *) 0, (offsetT) (n), (char *) 0);		\
     *p = 0x90;								\
     goto around;							\
   }
