@@ -1,6 +1,5 @@
 /* This file is tc-z8k.h
-
-   Copyright (C) 1987-1992 Free Software Foundation, Inc.
+   Copyright (C) 1987-1992, 93, 95, 1997 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -15,13 +14,18 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GAS; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with GAS; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 
 #define TC_Z8K
 #define TARGET_BYTES_BIG_ENDIAN 1
+
+#ifndef BFD_ASSEMBLER
 #define LOCAL_LABEL(x) 0
+#endif
+
 /* This macro translates between an internal fix and an coff reloc type */
 #define TC_COFF_FIX2RTYPE(fixP) abort();
 
@@ -33,7 +37,6 @@
 #define TC_RELOC_MANGLE(s,a,b,c) tc_reloc_mangle(a,b,c)
 
 #define DO_NOT_STRIP 0
-#define DO_STRIP 0
 #define LISTING_HEADER "Zilog Z8000 GAS "
 #define NEED_FX_R_TYPE 1
 #define RELOC_32 1234
