@@ -30,7 +30,7 @@
 #include "source.h"
 #include "sym_ids.h"
 
-#define VERSION "2.7.1"
+#define VERSION "2.8"
 
 const char *whoami;
 const char *function_mapping_file;
@@ -373,10 +373,9 @@ DEFUN (main, (argc, argv), int argc AND char **argv)
 	case 'v':
 	  /* This output is intended to follow the GNU standards document.  */
 	  printf ("GNU gprof %s\n", VERSION);
-	  printf ("Copyright 1996 Free Software Foundation, Inc.\n");
+	  printf ("Based on BSD gprof, copyright 1983 Regents of the University of California.\n");
 	  printf ("\
-This program is free software; you may redistribute it under the terms of\n\
-the GNU General Public License.  This program has absolutely no warranty.\n");
+This program is free software.  This program has absolutely no warranty.\n");
 	  done (0);
 	case 'w':
 	  output_width = atoi (optarg);
