@@ -1,5 +1,5 @@
 /* ldctor.c -- constructor support routines
-   Copyright (C) 1991, 92, 93, 94 Free Software Foundation, Inc.
+   Copyright (C) 1991, 92, 93, 94, 1995 Free Software Foundation, Inc.
    By Steve Chamberlain <sac@cygnus.com>
    
 This file is part of GLD, the Gnu Linker.
@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GLD; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "bfd.h"
 #include "sysdep.h"
@@ -34,27 +34,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    invoked by the command CONSTRUCTORS in the linker script.  */
 lang_statement_list_type constructor_list;
 
-/* We keep a list of these structures for each sets we build.  */
-
-struct set_info
-{
-  struct set_info *next;		/* Next set.  */
-  struct bfd_link_hash_entry *h;	/* Hash table entry.  */
-  bfd_reloc_code_real_type reloc;	/* Reloc to use for an entry.  */
-  size_t count;				/* Number of elements.  */
-  struct set_element *elements;		/* Elements in set.  */
-};
-
-struct set_element
-{
-  struct set_element *next;		/* Next element.  */
-  asection *section;			/* Section of value.  */
-  bfd_vma value;			/* Value.  */
-};
-
 /* The sets we have seen.  */
-
-static struct set_info *sets;
+struct set_info *sets;
 
 /* Add an entry to a set.  H is the entry in the linker hash table.
    RELOC is the relocation to use for an entry in the set.  SECTION
@@ -64,9 +45,10 @@ static struct set_info *sets;
    function will construct the sets.  */
 
 void
-ldctor_add_set_entry (h, reloc, section, value)
+ldctor_add_set_entry (h, reloc, name, section, value)
      struct bfd_link_hash_entry *h;
      bfd_reloc_code_real_type reloc;
+     const char *name;
      asection *section;
      bfd_vma value;
 {
@@ -88,11 +70,36 @@ ldctor_add_set_entry (h, reloc, section, value)
       p->count = 0;
       p->elements = NULL;
     }
-  else if (p->reloc != reloc)
-    einfo ("%P%X: Different relocs used in set %s\n", h->root.string);
+  else
+    {
+      if (p->reloc != reloc)
+	{
+	  einfo ("%P%X: Different relocs used in set %s\n", h->root.string);
+	  return;
+	}
+
+      /* Don't permit a set to be constructed from different object
+         file formats.  The same reloc may have different results.  We
+         actually could sometimes handle this, but the case is
+         unlikely to ever arise.  Sometimes constructor symbols are in
+         unusual sections, such as the absolute section--this appears
+         to be the case in Linux a.out--and in such cases we just
+         assume everything is OK.  */
+      if (p->elements != NULL
+	  && section->owner != NULL
+	  && p->elements->section->owner != NULL
+	  && strcmp (bfd_get_target (section->owner),
+		     bfd_get_target (p->elements->section->owner)) != 0)
+	{
+	  einfo ("%P%X: Different object file formats composing set %s\n",
+		 h->root.string);
+	  return;
+	}
+    }
 
   e = (struct set_element *) xmalloc (sizeof (struct set_element));
   e->next = NULL;
+  e->name = name;
   e->section = section;
   e->value = value;
 
@@ -111,24 +118,34 @@ ldctor_add_set_entry (h, reloc, section, value)
 void
 ldctor_build_sets ()
 {
+  static boolean called;
   lang_statement_list_type *old;
+  boolean header_printed;
   struct set_info *p;
 
+  /* The emulation code may call us directly, but we only want to do
+     this once.  */
+  if (called)
+    return;
+  called = true;
+
   old = stat_ptr;
   stat_ptr = &constructor_list;
 
   lang_list_init (stat_ptr);
 
+  header_printed = false;
   for (p = sets; p != (struct set_info *) NULL; p = p->next)
     {
       struct set_element *e;
-      const reloc_howto_type *howto;
+      reloc_howto_type *howto;
       int size;
 
       /* If the symbol is defined, we may have been invoked from
 	 collect, and the sets may already have been built, so we do
 	 not do anything.  */
-      if (p->h->type == bfd_link_hash_defined)
+      if (p->h->type == bfd_link_hash_defined
+	  || p->h->type == bfd_link_hash_defweak)
 	continue;
 
       /* For each set we build:
@@ -144,24 +161,40 @@ ldctor_build_sets ()
       howto = bfd_reloc_type_lookup (output_bfd, p->reloc);
       if (howto == (reloc_howto_type *) NULL)
 	{
-	  einfo ("%P%X: BFD backend does not support reloc %d for set %s\n",
-		 (int) p->reloc, p->h->root.string);
-	  size = LONG;
-	}
-      else
-	{
-	  switch (bfd_get_reloc_size (howto))
+	  if (link_info.relocateable)
 	    {
-	    case 1: size = BYTE; break;
-	    case 2: size = SHORT; break;
-	    case 4: size = LONG; break;
-	    case 8: size = QUAD; break;
-	    default:
-	      einfo ("%P%X: Unsupported size %d for set %s\n",
-		     bfd_get_reloc_size (howto), p->h->root.string);
-	      size = LONG;
-	      break;
+	      einfo ("%P%X: %s does not support reloc %s for set %s\n",
+		     bfd_get_target (output_bfd),
+		     bfd_get_reloc_code_name (p->reloc),
+		     p->h->root.string);
+	      continue;
 	    }
+
+	  /* If this is not a relocateable link, all we need is the
+	     size, which we can get from the input BFD.  */
+	  howto = bfd_reloc_type_lookup (p->elements->section->owner,
+					 p->reloc);
+	  if (howto == NULL)
+	    {
+	      einfo ("%P%X: %s does not support reloc %s for set %s\n",
+		     bfd_get_target (p->elements->section->owner),
+		     bfd_get_reloc_code_name (p->reloc),
+		     p->h->root.string);
+	      continue;
+	    }
+	}
+
+      switch (bfd_get_reloc_size (howto))
+	{
+	case 1: size = BYTE; break;
+	case 2: size = SHORT; break;
+	case 4: size = LONG; break;
+	case 8: size = QUAD; break;
+	default:
+	  einfo ("%P%X: Unsupported size %d for set %s\n",
+		 bfd_get_reloc_size (howto), p->h->root.string);
+	  size = LONG;
+	  break;
 	}
 
       lang_add_assignment (exp_assop ('=', p->h->root.string,
@@ -170,9 +203,39 @@ ldctor_build_sets ()
 
       for (e = p->elements; e != (struct set_element *) NULL; e = e->next)
 	{
+	  if (config.map_file != NULL)
+	    {
+	      int len;
+
+	      if (! header_printed)
+		{
+		  minfo ("\nSet                 Symbol\n\n");
+		  header_printed = true;
+		}
+
+	      minfo ("%s", p->h->root.string);
+	      len = strlen (p->h->root.string);
+
+	      if (len >= 19)
+		{
+		  print_nl ();
+		  len = 0;
+		}
+	      while (len < 20)
+		{
+		  print_space ();
+		  ++len;
+		}
+
+	      if (e->name != NULL)
+		minfo ("%T\n", e->name);
+	      else
+		minfo ("%G\n", e->section->owner, e->section, e->value);
+	    }
+
 	  if (link_info.relocateable)
-	    lang_add_reloc (p->reloc, howto, e->section,
-			    (const char *) NULL, exp_intop (e->value));
+	    lang_add_reloc (p->reloc, howto, e->section, e->name,
+			    exp_intop (e->value));
 	  else
 	    lang_add_data (size, exp_relop (e->section, e->value));
 	}
