@@ -1,5 +1,6 @@
 /* Print i386 instructions for GDB, the GNU debugger.
-   Copyright (C) 1988, 89, 91, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1988, 89, 91, 93, 94, 95, 96, 1997
+   Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -39,6 +40,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include <setjmp.h>
 
+static int fetch_data PARAMS ((struct disassemble_info *, bfd_byte *));
+
 struct dis_private
 {
   /* Points to first byte not fetched.  */
@@ -78,9 +81,6 @@ fetch_data (info, addr)
   return 1;
 }
 
-#define Em OP_E, m_mode
-#define Gm OP_G, m_mode
-
 #define Eb OP_E, b_mode
 #define indirEb OP_indirE, b_mode
 #define Gb OP_G, b_mode
@@ -180,7 +180,6 @@ static void ckprefix PARAMS ((void));
 #define v_mode 2
 #define w_mode 3
 #define d_mode 4
-#define m_mode 5
 
 #define es_reg 100
 #define cs_reg 101
@@ -236,9 +235,6 @@ static void ckprefix PARAMS ((void));
 #define GRP7 NULL, NULL, 14
 #define GRP8 NULL, NULL, 15
 #define GRP9 NULL, NULL, 16
-#define GRP10_71 NULL, NULL, 17
-#define GRP10_72 NULL, NULL, 18
-#define GRP10_73 NULL, NULL, 19
 
 #define FLOATCODE 50
 #define FLOAT NULL, NULL, FLOATCODE
@@ -597,41 +593,17 @@ static struct dis386 dis386_twobyte[] = {
   { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* 60 */
-  { "punpcklbw", Gm, Em },  
-  { "punpcklwd", Gm, Em },  
-  { "punpckldq", Gm, Em },  
-  { "packsswb", Gm, Em },  
-  { "pcmpgtb", Gm, Em },  
-  { "pcmpgtw", Gm, Em },  
-  { "pcmpgtd", Gm, Em },  
-  { "packuswb", Gm, Em },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* 68 */
-  { "punpckhbw", Gm, Em },  
-  { "punpckhwd", Gm, Em },  
-  { "punpckhdq", Gm, Em },  
-  { "packssdw", Gm, Em },  
-  { "(bad)" }, 
-  { "(bad)" },  
-  { "movd", Gm, Ev },  
-  { "movq", Gm, Em },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* 70 */
-  { "(bad)" }, 
-  { GRP10_71 }, 
-  { GRP10_72 },
-  { GRP10_73 },  
-  { "pcmpeqb", Gm, Em }, 
-  { "pcmpeqw", Gm, Em }, 
-  { "pcmpeqd", Gm, Em },  
-  { "emms" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* 78 */
-  { "(bad)" },  
-  { "(bad)" },  
-  { "(bad)" }, 
-  { "(bad)" },  
-  { "(bad)" }, 
-  { "(bad)" }, 
-  { "movd", Ev, Gm }, 
-  { "movq", Em, Gm },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* 80 */
   { "jo", Jv },
   { "jno", Jv },
@@ -723,59 +695,23 @@ static struct dis386 dis386_twobyte[] = {
   { "bswap", eSI },
   { "bswap", eDI },
   /* d0 */
-  { "(bad)" }, 
-  { "psrlw", Gm, Em }, 
-  { "psrld", Gm, Em }, 
-  { "psrlq", Gm, Em }, 
-  { "(bad)" },
-  { "pmullw", Gm, Em },
-  { "(bad)" }, 
-  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* d8 */
-  { "psubusb", Gm, Em }, 
-  { "psubusw", Gm, Em }, 
-  { "(bad)" },
-  { "pand", Gm, Em },  
-  { "paddusb", Gm, Em }, 
-  { "paddusw", Gm, Em }, 
-  { "(bad)" }, 
-  { "pandn", Gm, Em },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* e0 */
-  { "(bad)" }, 
-  { "psraw", Gm, Em }, 
-  { "psrad", Gm, Em },  
-  { "(bad)" },  
-  { "(bad)" }, 
-  { "pmulhw", Gm, Em }, 
-  { "(bad)" }, 
-  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* e8 */
-  { "psubsb", Gm, Em },  
-  { "psubsw", Gm, Em },  
-  { "(bad)" },  
-  { "por", Gm, Em },  
-  { "paddsb", Gm, Em }, 
-  { "paddsw", Gm, Em },  
-  { "(bad)" },  
-  { "pxor", Gm, Em},  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* f0 */
-  { "(bad)" },  
-  { "psllw", Gm, Em }, 
-  { "pslld", Gm, Em }, 
-  { "psllq", Gm, Em },  
-  { "(bad)" }, 
-  { "pmaddwd", Gm, Em }, 
-  { "(bad)" }, 
-  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
   /* f8 */
-  { "psubb", Gm, Em },  
-  { "psubw", Gm, Em },  
-  { "psubq", Gm, Em },  
-  { "(bad)" },  
-  { "paddb", Gm, Em }, 
-  { "paddw", Gm, Em }, 
-  { "paddd", Gm, Em },  
-  { "ud2" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
+  { "(bad)" },  { "(bad)" },  { "(bad)" },  { "(bad)" },  
 };
 
 static const unsigned char onebyte_has_modrm[256] = {
@@ -827,9 +763,6 @@ static int rm;
 static int reg;
 static void oappend PARAMS ((char *s));
 
-static char *namesmm[]={
-  "%mm0","%mm1","%mm2","%mm3", "%mm4","%mm5","%mm6","%mm7",
-};
 static char *names32[]={
   "%eax","%ecx","%edx","%ebx", "%esp","%ebp","%esi","%edi",
 };
@@ -1033,39 +966,6 @@ static struct dis386 grps[][8] = {
     { "(bad)" },
     { "(bad)" },
     { "(bad)" },
-  },
-  /* GRP10_71 */
-  {
-    { "(bad)" },
-    { "(bad)" },
-    { "psrlw", Em, Ib },
-    { "(bad)" },
-    { "psraw", Em, Ib },
-    { "(bad)" },
-    { "psllw", Em, Ib },
-    { "(bad)" },
-  },
-  /* GRP10_72 */
-  {
-    { "(bad)" },
-    { "(bad)" },
-    { "psrld", Em, Ib },
-    { "(bad)" },
-    { "psrad", Em, Ib },
-    { "(bad)" },
-    { "pslld", Em, Ib },
-    { "(bad)" },
-  },
-  /* GRP10_73 */
-  {
-    { "(bad)" },
-    { "(bad)" },
-    { "psrlq", Em, Ib },
-    { "(bad)" },
-    { "(bad)" },
-    { "(bad)" },
-    { "psllq", Em, Ib },
-    { "(bad)" },
   }
 };
 
@@ -1502,10 +1402,10 @@ static struct dis386 float_reg[][8] = {
     { "fmulp",	STi, ST },
     { "(bad)" },
     { FGRPde_3 },
-    { "fsubp",	STi, ST },
     { "fsubrp",	STi, ST },
-    { "fdivp",	STi, ST },
+    { "fsubp",	STi, ST },
     { "fdivrp",	STi, ST },
+    { "fdivp",	STi, ST },
   },
   /* df */
   {
@@ -1733,9 +1633,6 @@ OP_E (bytemode, aflag, dflag)
 	  else
 	    oappend (names16[rm]);
 	  break;
-	case m_mode:
-	  oappend (namesmm[rm]);
-	  break;
 	default:
 	  oappend ("<bad dis table>");
 	  break;
@@ -1866,9 +1763,6 @@ OP_G (bytemode, aflag, dflag)
       else
 	oappend (names16[reg]);
       break;
-    case m_mode:
-      oappend (namesmm[reg]);
-      break;
     default:
       oappend ("<internal disassembler error>");
       break;
@@ -2024,11 +1918,6 @@ OP_J (bytemode, aflag, dflag)
     case b_mode:
       FETCH_DATA (the_info, codep + 1);
       disp = *(char *)codep++;
-
-      /* sign extend number */
-      if (disp >= 1<<7) {
-	disp -= 1<<8;
-      }
       break;
     case v_mode:
       if (dflag)
@@ -2036,10 +1925,6 @@ OP_J (bytemode, aflag, dflag)
       else
 	{
 	  disp = (short)get16 ();
-	  /* sign extend number */
-	  if (disp >= 1<<15) {
-	    disp -= 1<<16;
-	  }
 	  /* for some reason, a data16 prefix on a jump instruction
 	     means that the pc is masked to 16 bits after the
 	     displacement is added!  */
