@@ -1,5 +1,5 @@
 /* Print SPARC instructions.
-   Copyright (C) 1989, 91-93, 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1989, 91-94, 1995, 1996, 1997 Free Software Foundation, Inc.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,11 +15,13 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+#include <stdio.h>
+
 #include "ansidecl.h"
+#include "sysdep.h"
 #include "opcode/sparc.h"
 #include "dis-asm.h"
 #include "libiberty.h"
-#include <string.h>
 
 /* Bitmask of v9 architectures.  */
 #define MASK_V9 ((1 << SPARC_OPCODE_ARCH_V9) \
@@ -44,7 +46,12 @@ struct opcode_hash {
   struct sparc_opcode *opcode;
 };
 static struct opcode_hash *opcode_hash_table[HASH_SIZE];
-static void build_hash_table ();
+
+static void build_hash_table
+  PARAMS ((struct sparc_opcode *, struct opcode_hash **, int));
+static int is_delayed_branch PARAMS ((unsigned long));
+static int compare_opcodes PARAMS ((const PTR, const PTR));
+static int compute_arch_mask PARAMS ((unsigned long));
 
 /* Sign-extend a value which is N bits long.  */
 #define	SEX(value, bits) \
@@ -177,12 +184,10 @@ is_delayed_branch (insn)
 }
 
 /* extern void qsort (); */
-static int compare_opcodes ();
 
 /* Records current mask of SPARC_OPCODE_ARCH_FOO values, used to pass value
    to compare_opcodes.  */
 static unsigned int current_arch_mask;
-static int compute_arch_mask ();
 
 /* Print one instruction from MEMADDR on INFO->STREAM.
 
@@ -434,7 +439,7 @@ print_insn_sparc (memaddr, info)
 		    {
 		      int mask = X_MEMBAR (insn);
 		      int bit = 0x40, printed_one = 0;
-		      char *name;
+		      const char *name;
 
 		      if (mask == 0)
 			(info->fprintf_func) (stream, "0");
@@ -519,7 +524,7 @@ print_insn_sparc (memaddr, info)
 
 		  case '*':
 		    {
-		      char *name = sparc_decode_prefetch (X_RD (insn));
+		      const char *name = sparc_decode_prefetch (X_RD (insn));
 
 		      if (name)
 			(*info->fprintf_func) (stream, "%s", name);
@@ -553,7 +558,7 @@ print_insn_sparc (memaddr, info)
 
 		  case 'A':
 		    {
-		      char *name = sparc_decode_asi (X_ASI (insn));
+		      const char *name = sparc_decode_asi (X_ASI (insn));
 
 		      if (name)
 			(*info->fprintf_func) (stream, "%s", name);
@@ -604,7 +609,7 @@ print_insn_sparc (memaddr, info)
 		  case 'U':
 		    {
 		      int val = *s == 'U' ? X_RS1 (insn) : X_RD (insn);
-		      char *name = sparc_decode_sparclet_cpreg (val);
+		      const char *name = sparc_decode_sparclet_cpreg (val);
 
 		      if (name)
 			(*info->fprintf_func) (stream, "%s", name);
@@ -731,7 +736,8 @@ compute_arch_mask (mach)
 
 static int
 compare_opcodes (a, b)
-     char *a, *b;
+     const PTR a;
+     const PTR b;
 {
   struct sparc_opcode *op0 = (struct sparc_opcode *) a;
   struct sparc_opcode *op1 = (struct sparc_opcode *) b;
